diff -urNX /home/leigh/.diffex linux-2.4.22-prev/arch/ppc/platforms/prep_setup.c linux-2.4.22/arch/ppc/platforms/prep_setup.c
--- linux-2.4.22-prev/arch/ppc/platforms/prep_setup.c	Mon Aug 25 16:48:01 2003
+++ linux-2.4.22/arch/ppc/platforms/prep_setup.c	Mon Aug 25 16:53:34 2003
@@ -691,6 +691,123 @@
 #endif /* CONFIG_PREP_RESIDUAL */
 }
 
+/*
+ * Set BAT 2 to access 0x80000000 so early progress messages will work
+ */
+static __inline__ void
+prep_set_bat(void)
+{
+	static int mapping_set = 0;
+
+	if (!mapping_set) {
+		/* wait for all outstanding memory access to complete */
+		mb();
+
+		/* setup DBATs */
+		mtspr(DBAT2U, 0x80001ffe);
+		mtspr(DBAT2L, 0x8000002a);
+
+		/* wait for updates */
+		mb();
+
+		mapping_set = 1;
+	}
+}
+
+/*
+ * IBM 3-digit status LED - only if we have PReP residual data to find it with
+ */
+#ifdef CONFIG_PREP_RESIDUAL
+static unsigned int ibm_statusled_base __prepdata;
+
+static void __prep
+ibm_statusled_progress(char *s, unsigned short hex);
+
+static int __prep
+ibm_statusled_panic(struct notifier_block *dummy1, unsigned long dummy2,
+		    void * dummy3)
+{
+	ibm_statusled_progress(NULL, 0x505); /* SOS (suggestions welcome) */
+	return NOTIFY_DONE;
+}
+
+static struct notifier_block ibm_statusled_block __prepdata = {
+	ibm_statusled_panic,
+	NULL,
+	INT_MAX /* try to do it first */
+};
+
+static void __prep
+ibm_statusled_progress(char *s, unsigned short hex)
+{
+	static int notifier_installed;
+	/*
+	 * Progress uses 4 digits and we have only 3.  So, we map 0xffff to
+	 * 0xfff for display switch off.  Out of range values are mapped to
+	 * 0xeff, as I'm told 0xf00 and above are reserved for hardware codes.
+	 * Install the panic notifier when the display is first switched off.
+	 */
+	if (hex == 0xffff) {
+		hex = 0xfff;
+		if (!notifier_installed) {
+			++notifier_installed;
+			notifier_chain_register(&panic_notifier_list,
+						&ibm_statusled_block);
+		}
+	}
+	else
+		if (hex > 0xfff)
+			hex = 0xeff;
+
+	/* A small delay seems to be needed - purely guesswork */
+	outb(hex >> 0x08, ibm_statusled_base + 1);
+	udelay(100);
+	outb(hex  & 0xff, ibm_statusled_base);
+	udelay(200);
+}
+
+static void __init
+ibm_statusled_init(void)
+{
+	PPC_DEVICE *leddevice;
+	PnP_TAG_PACKET *pkt;
+
+	/*
+	 * The IBM 3-digit LED display is specified in the residual data
+	 * as an operator panel device, type "System Status LED".  Find
+	 * that device and determine its address.  We validate all the
+	 * other parameters on the off-chance another, similar device
+	 * exists.
+	 */
+	leddevice = residual_find_device(~0, NULL, SystemPeripheral,
+					 OperatorPanel, SystemStatusLED, 0);
+	if (!leddevice)
+		return;
+
+	pkt = PnP_find_packet((unsigned char *)&res->DevicePnPHeap[leddevice->AllocatedOffset], S8_Packet, 0);
+	if (!pkt)
+		return;
+
+	if (pkt->S8_Pack.IOInfo != ISAAddr16bit)
+		return;
+	if (*(unsigned short *)pkt->S8_Pack.RangeMin !=
+	    *(unsigned short *)pkt->S8_Pack.RangeMax)
+		return;
+	if (pkt->S8_Pack.IOAlign != 2)
+		return;
+	if (pkt->S8_Pack.IONum != 2)
+		return;
+
+	ibm_statusled_base = ld_le16( (unsigned short *)(pkt->S8_Pack.RangeMin));
+	ppc_md.progress = ibm_statusled_progress;
+}
+#else /* !CONFIG_PREP_RESIDUAL */
+static void __init
+ibm_statusled_init(void)
+{
+}
+#endif /* CONFIG_PREP_RESIDUAL */
+
 static void __init
 prep_setup_arch(void)
 {
@@ -1183,6 +1300,10 @@
 		ppc_md.set_rtc_time   = mc146818_set_rtc_time;
 		ppc_md.get_rtc_time   = mc146818_get_rtc_time;
 		ppc_md.calibrate_decr = prep_calibrate_decr;
+
+		/* Initialise progress early to get maximum benefit */
+		prep_set_bat();
+		ibm_statusled_init();
 	} else {
 		ppc_md.set_rtc_time   = mk48t59_set_rtc_time;
 		ppc_md.get_rtc_time   = mk48t59_get_rtc_time;
